// SPDX-License-Identifier: GPL-2.0-only
/*
* alap_rpc.c -- ALAP Remote Procedure Call Communication Driver
*
* Copyright (C) 2016-2018 ALPINE ELECTRONICS, INC.
* Copyright (C) 2019 ALPSALPINE CO.,LTD.
*/
/*
 * ---------------------------------------------------------
 * File Name    : alap_rpc.c
 * Date         : 10 Mar 2021
 * Author       : L.X
 * Model        : 30AA
 * Description  : Add for 30AA rpc spi driver
 * File Version : 0.00
 * ---------------------------------------------------------
 * Revision History
 * No       Date        Revised by      Function Name
 * ---------------------------------------------------------
 * 0.00     10 Mar 2021 L.X             * creat module
 * ---------------------------------------------------------
 */

#include <linux/init.h>
#include <linux/module.h>
#include <linux/ioctl.h>
#include <linux/fs.h>
#include <linux/device.h>
#include <linux/err.h>
#include <linux/list.h>
#include <linux/errno.h>
#include <linux/mutex.h>
#include <linux/slab.h>
#include <linux/compat.h>
#include <linux/of.h>
#include <linux/of_device.h>
#include <linux/acpi.h>
#include <linux/spi/spi.h>
#include <linux/irq.h>
#include <linux/interrupt.h>
#include <linux/gpio/consumer.h>
#include <linux/pinctrl/consumer.h>
#include <linux/uaccess.h>
#include <linux/kernel.h>
#include <linux/syscalls.h>
/*
 * -----------------------------------
 * Macro Definition Section
 * -----------------------------------
 */
#define ALAP_RPC_MAJOR				200 /* assigned */
#define ALAP_RPC_N_SPI_MINORS		32  /* ... up to 256 */

static DECLARE_BITMAP(minors, ALAP_RPC_N_SPI_MINORS);

#define DEVICE_NAME "alap_rpc"

#define GPIO_LOW 0
#define GPIO_HIGH 1

/* Configuration for probe S */
#define ALAP_RPC_WT_COUNT 512
#define ALAP_RPC_RD_COUNT 512
#define ALAP_RPC_WT_LEN 124
#define ALAP_RPC_RD_LEN 128
#define ALAP_RPC_DATA_SIZE 128
#define ALAP_RPC_RETRY_MAX 3
#define ALAP_RPC_PROC_BLOCK 0
#define ALAP_RPC_DATA_COMBI (1)    /* Enable compose message to frame :  1 (enable) , 0 (disable) */
#define ALAP_RPC_DATA_SPLIT (0)    /* Enable split frame to messgae :  1 (enable) , 0 (disable) */
/* Configuration for probe E */

/* Configuration for test S */
#define RPC_DEBUG_LEVEL 0
/* Configuration for test E */

#define ALAP_RPC_FRAME_SIZE 4
#define ALAP_RPC_INDX_PKST 0
#define ALAP_RPC_INDX_NUMB 1
#define ALAP_RPC_INDX_DAST 2
#define ALAP_RPC_OFFS_CSUM 2
#define ALAP_RPC_OFFS_PKED 3

#define ALAP_RPC_ACK_SIZE 1
#define ALAP_RPC_ACK_DATA 0
#define ALAP_RPC_ACK  0x06
#define ALAP_RPC_NACK 0x15
#define ALAP_RPC_SFRAM 0x01
#define ALAP_RPC_NONUM 0x00
#define ALAP_RPC_EFRAM 0x17

#define ALAP_RPC_SEND_DATA 1
#define ALAP_RPC_SEND_ACK  2

#define ALAP_RPC_DEMPTY_FLAG  1
#define ALAP_RPC_DNORMAL_FLAG 2

#define ALAP_RPC_WQ_NONE 0
#define ALAP_RPC_WQ_WAIT 1
#define ALAP_RPC_WQ_WAKE 2

#define ALAP_RPC_STR_NONE 0
#define ALAP_RPC_STR_WAIT 1
#define ALAP_RPC_STR_WAKE 2

#define ALAP_RPC_NO_TRANS 0
#define ALAP_RPC_TRANSING 1

#define ALAP_RPC_MAXSPEEDHZ (4915200)

/*
 * -----------------------------------
 * Structure Type Definition Section
 * -----------------------------------
 */

struct point_bit {
	unsigned int page_bit :  1;
	unsigned int index_c  :  31;
};

union index {
	unsigned int		index;
	struct point_bit	bit;
};

struct rpc_b {
	unsigned int		bflag;
	wait_queue_head_t   wqueue;
	unsigned int		condi;
};

struct rpc_sendf {
	unsigned int		process_flag;
	unsigned int		Empty_data_flag;
	unsigned int		combi_count;
};

struct rpc_retry {
	unsigned int		count;
	unsigned char		*rty_buf;
};

struct alap_rpc_data {
	/* device */
	dev_t				devt;
	spinlock_t			spi_lock;
	struct spi_device	*spi_dev;
	struct list_head	device_entry;
	unsigned int		users;

	/* kernel process */
	struct gpio_desc	*startup_gpiod;
	unsigned char		*tx_tmpb;
	unsigned char		*rx_tmpb;
	unsigned int		trans_len;
	unsigned int		trans_status;
	struct rpc_sendf	send_flag;
	struct rpc_retry	retry;

	struct rpc_b		rpc_suspend;

	/* user process */
	struct rpc_b		wt_block;
	struct mutex		wt_buf_lock;
	unsigned char		*wt_buf;
	unsigned int		wt_len;
	union index			wt_rd;
	union index			wt_wt;
	unsigned int		wt_buf_cont;

	struct rpc_b		rd_block;
	struct mutex		rd_buf_lock;
	unsigned char		*rd_buf;
	unsigned int		rd_len;
	union index			rd_rd;
	union index			rd_wt;
	unsigned int		rd_buf_cont;
};

static LIST_HEAD(device_list);
static DEFINE_MUTEX(device_list_lock);

const unsigned char empdata_buf[ALAP_RPC_DATA_SIZE] = 
{
	0x01, 0x00, 																					// SOH, Num of Data
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // Data Body #0~#15
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // Data Body #16~#31
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // Data Body #32~#47
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // Data Body #48~#63
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // Data Body #64~#79
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // Data Body #80~#95
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // Data Body #96~#111
	0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 						// Data Body #112~#123
	0x01, 0x17																						// Check Sum, ETB
};

/*
 * -----------------------------------
 * rpc performance Statistics.
 * -----------------------------------
 */
static unsigned int rpc_tdebug;
module_param(rpc_tdebug, uint, 0600);
MODULE_PARM_DESC(rpc_tdebug, "Testdebug switch for rpc debug");

#define ALAP_DEBUG_L1(str, args...) do {\
	if (rpc_tdebug > 0)\
		dev_info(&spi->dev, str, ##args);\
} while (0)\

#define ALAP_DEBUG_L2(str, args...) do {\
	if (rpc_tdebug > 1)\
		dev_info(&spi->dev, str, ##args);\
} while (0)\

/*
 * -----------------------------------
 * Function Definition Section
 * -----------------------------------
 */

static unsigned char
alap_rpc_calc_checksum(unsigned char *data, unsigned int size)
{
	unsigned int	loop_cnt;
	unsigned char	calc_data;

	calc_data = 0;

	if (data != NULL) {
		for (loop_cnt = 0; loop_cnt < size; loop_cnt++)
			calc_data = calc_data + data[loop_cnt];
	}

	return calc_data;
}

static bool
alap_rpc_calc_checksum_chk(struct spi_device *spi,
	unsigned char *data, unsigned int size, unsigned char incs)
{
	unsigned char	calc_data;

	calc_data = alap_rpc_calc_checksum(data, size);
	if (incs == calc_data)
		return TRUE;

	return FALSE;
}

static ssize_t
alap_rpc_sync(struct alap_rpc_data *alap_rpc, int len)
{
	struct	spi_device *spi = alap_rpc->spi_dev;
	int		status;

	struct spi_transfer	t = {
		.tx_buf		= alap_rpc->tx_tmpb,
		.rx_buf		= alap_rpc->rx_tmpb,
		.len		= len,
		.cs_change	= 0,
		};
	struct spi_message	m;

	spi_message_init(&m);
	spi_message_add_tail(&t, &m);

	if (spi == NULL)
		status = -ESHUTDOWN;
	else
		status = spi_sync(spi, &m);

	if (status == 0)
		status = m.actual_length;

	return status;
}
static void alap_rpc_tx_data_combi(struct alap_rpc_data *alap_rpc)
{
	struct spi_device *spi = alap_rpc->spi_dev;
	unsigned int tmplen;
	unsigned int combicont = 0;
	unsigned int TmpbInd;
	unsigned int TmpwtInd_c;

	/* tx buffer clear */
	tmplen = alap_rpc->trans_len;
	memset(alap_rpc->tx_tmpb, 0x00, tmplen);
	/* Data make perpare */
	TmpwtInd_c = alap_rpc->wt_rd.bit.index_c;
	TmpbInd =  TmpwtInd_c * alap_rpc->wt_len;
	/* make message head */
	alap_rpc->tx_tmpb[ALAP_RPC_INDX_PKST] = ALAP_RPC_SFRAM;

	#if ALAP_RPC_DATA_COMBI
	{
		unsigned int loopStart = 0;
	
		while (1) {
			if (loopStart == 0)
				loopStart = ALAP_RPC_INDX_DAST;
			else
				loopStart = loopStart + tmplen;

			TmpbInd =  TmpwtInd_c * alap_rpc->wt_len;
			tmplen = alap_rpc->wt_buf[TmpbInd] + 1;
			if ((tmplen + loopStart > alap_rpc->trans_len)
			 || (tmplen == 1))
				break;
			memcpy(&(alap_rpc->tx_tmpb[loopStart]),
			   &(alap_rpc->wt_buf[TmpbInd]), tmplen);

			ALAP_DEBUG_L1(
			   "tx combi: SIndex(%d),combicont(%d),wt_rd(%d),wt_wt(%d),len(%d)\n",
			   loopStart,
			   combicont,
			   alap_rpc->wt_wt.bit.index_c,
			   TmpwtInd_c,
			   tmplen);

			ALAP_DEBUG_L1(
			   "tx combi: tmdata(%x)(%x)(%x)(%x)(%x)..(%x)\n",
			   alap_rpc->tx_tmpb[loopStart],
			   alap_rpc->tx_tmpb[loopStart + 1],
			   alap_rpc->tx_tmpb[loopStart + 2],
			   alap_rpc->tx_tmpb[loopStart + 3],
			   alap_rpc->tx_tmpb[loopStart + 4],
			   alap_rpc->tx_tmpb[loopStart + alap_rpc->wt_len - 1]);

			ALAP_DEBUG_L1(
			   "tx combi: Ind(%d)wtdata(%x)(%x)(%x)(%x)(%x)..(%x)\n",
			   TmpbInd,
			   alap_rpc->wt_buf[TmpbInd],
			   alap_rpc->wt_buf[TmpbInd + 1],
			   alap_rpc->wt_buf[TmpbInd + 2],
			   alap_rpc->wt_buf[TmpbInd + 3],
			   alap_rpc->wt_buf[TmpbInd + 4],
			   alap_rpc->wt_buf[TmpbInd + alap_rpc->wt_len - 1]);

			TmpwtInd_c++;
			if (TmpwtInd_c == alap_rpc->wt_buf_cont)
				TmpwtInd_c = 0;
			combicont++;

			if (alap_rpc->wt_wt.bit.index_c == TmpwtInd_c)
				break;
		};
	}
	#else /* ALAP_RPC_DATA_COMBI */
		combicont = 1;
		tmplen = alap_rpc->wt_buf[TmpbInd] + 1;

		ALAP_DEBUG_L1(
			"tx no combi: Ind(%d)wtdata(%x)(%x)(%x)(%x)(%x)..(%x)\n",
			TmpbInd,
			alap_rpc->wt_buf[TmpbInd],
			alap_rpc->wt_buf[TmpbInd + 1],
			alap_rpc->wt_buf[TmpbInd + 2],
			alap_rpc->wt_buf[TmpbInd + 3],
			alap_rpc->wt_buf[TmpbInd + 4],
			alap_rpc->wt_buf[TmpbInd + alap_rpc->wt_len - 1]);

		memcpy(&(alap_rpc->tx_tmpb[ALAP_RPC_INDX_DAST]),
		   &(alap_rpc->wt_buf[TmpbInd]), alap_rpc->wt_len);
	#endif  /* ALAP_RPC_DATA_COMBI */

	/* make number of data */
	alap_rpc->tx_tmpb[ALAP_RPC_INDX_NUMB] = combicont;
	/* make checksum */
	tmplen = alap_rpc->trans_len - ALAP_RPC_FRAME_SIZE;

	alap_rpc->tx_tmpb[tmplen + ALAP_RPC_OFFS_CSUM] =
	   alap_rpc_calc_checksum(alap_rpc->tx_tmpb,
	   alap_rpc->trans_len - ALAP_RPC_OFFS_CSUM);
	/* make message end */
	alap_rpc->tx_tmpb[tmplen + ALAP_RPC_OFFS_PKED] = ALAP_RPC_EFRAM;
	alap_rpc->send_flag.combi_count = combicont;
}

static bool alap_rpc_read_buf_chk(struct alap_rpc_data *alap_rpc)
{
	/* read buffer buff check */
	if ((alap_rpc->rd_wt.bit.index_c == alap_rpc->rd_rd.bit.index_c)
	 && (alap_rpc->rd_wt.bit.page_bit !=
	   alap_rpc->rd_rd.bit.page_bit))
		return false;

	/* After split, read buffer override check */
	#if ALAP_RPC_DATA_SPLIT
		if (alap_rpc->rx_tmpb[ALAP_RPC_INDX_NUMB] != 1) {
			unsigned int splitcont;
			splitcont = alap_rpc->rx_tmpb[ALAP_RPC_INDX_NUMB];
			if (alap_rpc->rd_wt.bit.page_bit !=
			   alap_rpc->rd_rd.bit.page_bit) {
				if (alap_rpc->rd_wt.bit.index_c + splitcont >
				   alap_rpc->rd_rd.bit.index_c)
					return false;
			}
		}
	#endif /* ALAP_RPC_DATA_SPLIT */
	return true;
}

static int alap_rpc_rx_data_split(struct alap_rpc_data *alap_rpc)
{
	struct spi_device *spi = alap_rpc->spi_dev;
	unsigned int tmplen;
	unsigned int splitcont;
	unsigned int loopStart = 0;
	unsigned int tmpbIndex;
	unsigned int TmpwtInd_c;

	splitcont = alap_rpc->rx_tmpb[ALAP_RPC_INDX_NUMB];
	tmplen = alap_rpc->rx_tmpb[ALAP_RPC_INDX_DAST] + 1;
	TmpwtInd_c = alap_rpc->rd_wt.bit.index_c;

	ALAP_DEBUG_L1("rx split: splitcont (%d)\n",
	   splitcont);

	#if ALAP_RPC_DATA_SPLIT
	{
		unsigned int loopind = 0;
		while (1) {
			loopind++;
			if (loopStart == 0)
				loopStart = ALAP_RPC_INDX_DAST;
			else {
				loopStart = loopStart + tmplen;
				tmplen = alap_rpc->rx_tmpb[loopStart] + 1;
			}

			if (loopind > splitcont)
				break;

			if ((loopind <= splitcont)
			 && (loopStart + tmplen > alap_rpc->trans_len))
				return 1;

			if (tmplen == 1)
				return 1;

			tmpbIndex =
			   TmpwtInd_c * alap_rpc->rd_len;
			memcpy(&(alap_rpc->rd_buf[tmpbIndex]),
			   &(alap_rpc->rx_tmpb[loopStart]), tmplen);

			ALAP_DEBUG_L1(
			   "rx split: SIndex(%d), loopInd(%d), rd_wt(%d), len(%d)\n",
			   loopStart,
			   loopind,
			   TmpwtInd_c,
			   tmplen);

			ALAP_DEBUG_L1(
			   "rx split: tmpdata(%x)(%x)(%x)(%x)(%x)..(%x)\n",
			   alap_rpc->rx_tmpb[loopStart],
			   alap_rpc->rx_tmpb[loopStart + 1],
			   alap_rpc->rx_tmpb[loopStart + 2],
			   alap_rpc->rx_tmpb[loopStart + 3],
			   alap_rpc->rx_tmpb[loopStart + 4],
			   alap_rpc->rx_tmpb[loopStart + tmplen - 1]);

			ALAP_DEBUG_L1(
			   "rx split: Ind(%d)rddata(%x)(%x)(%x)(%x)(%x)..(%x)\n",
			   tmpbIndex,
			   alap_rpc->rd_buf[tmpbIndex],
			   alap_rpc->rd_buf[tmpbIndex + 1],
			   alap_rpc->rd_buf[tmpbIndex + 2],
			   alap_rpc->rd_buf[tmpbIndex + 3],
			   alap_rpc->rd_buf[tmpbIndex + 4],
			   alap_rpc->rd_buf[tmpbIndex + tmplen - 1]);

			TmpwtInd_c++;
			if (TmpwtInd_c == alap_rpc->rd_buf_cont)
				TmpwtInd_c = 0;
		};
	}
	#else /* ALAP_RPC_DATA_SPLIT */
		splitcont = 1;
		tmplen = alap_rpc->rd_len;
		loopStart = 0;
		tmpbIndex =
		   TmpwtInd_c * alap_rpc->rd_len;
		memcpy(&(alap_rpc->rd_buf[tmpbIndex]),
		   &(alap_rpc->rx_tmpb[loopStart]), tmplen);
	#endif /* ALAP_RPC_DATA_SPLIT */

	alap_rpc->rd_wt.bit.index_c += splitcont;
	if (alap_rpc->rd_wt.bit.index_c >=
	   alap_rpc->rd_buf_cont) {
		alap_rpc->rd_wt.bit.index_c =
		   alap_rpc->rd_wt.bit.index_c - alap_rpc->rd_buf_cont;
		alap_rpc->rd_wt.bit.page_bit =
		   ~alap_rpc->rd_wt.bit.page_bit;
	}

	return 0;
}

static void alap_rpc_data_mesgchk(struct alap_rpc_data *alap_rpc)
{
	struct spi_device *spi = alap_rpc->spi_dev;
	ssize_t status = 0;
	bool checkcs;
	int ret;

	/*
	 * Only this function can operation
	 * tx_read and rx_write point
	 */

	alap_rpc->send_flag.Empty_data_flag = ALAP_RPC_DNORMAL_FLAG;
	/* check tx data */
	if ((alap_rpc->wt_rd.bit.index_c == alap_rpc->wt_wt.bit.index_c)
	 && (alap_rpc->wt_rd.bit.page_bit ==
		   alap_rpc->wt_wt.bit.page_bit)) {
		/* make empty buffer */
		memcpy(alap_rpc->tx_tmpb,
		   empdata_buf,
		   alap_rpc->trans_len);
		alap_rpc->send_flag.Empty_data_flag = ALAP_RPC_DEMPTY_FLAG;
	} else {
		if (alap_rpc->retry.count == 0)
			alap_rpc_tx_data_combi(alap_rpc);
		else
			memcpy(alap_rpc->tx_tmpb,
			   alap_rpc->retry.rty_buf,
			   alap_rpc->trans_len);
	}

	/* communication data */
	status = alap_rpc_sync(alap_rpc, alap_rpc->trans_len);

	if ((alap_rpc->tx_tmpb[1] != 0) && (rpc_tdebug != 2))
		ALAP_DEBUG_L1(
		   "tx data: len(%d),data:(%x)(%x)(%x)(%x)(%x)(%x)(%x)..(%x)\n",
		   alap_rpc->trans_len,
		   alap_rpc->tx_tmpb[0],
		   alap_rpc->tx_tmpb[1],
		   alap_rpc->tx_tmpb[2],
		   alap_rpc->tx_tmpb[3],
		   alap_rpc->tx_tmpb[4],
		   alap_rpc->tx_tmpb[5],
		   alap_rpc->tx_tmpb[6],
		   alap_rpc->tx_tmpb[alap_rpc->trans_len - 2]);

	if ((alap_rpc->rx_tmpb[1] != 0) && (rpc_tdebug != 2))
		ALAP_DEBUG_L1(
		   "rx data: len(%ld),data:(%x)(%x)(%x)(%x)(%x)(%x)(%x)..(%x)\n",
		   status,
		   alap_rpc->rx_tmpb[0],
		   alap_rpc->rx_tmpb[1],
		   alap_rpc->rx_tmpb[2],
		   alap_rpc->rx_tmpb[3],
		   alap_rpc->rx_tmpb[4],
		   alap_rpc->rx_tmpb[5],
		   alap_rpc->rx_tmpb[6],
		   alap_rpc->rx_tmpb[alap_rpc->trans_len - 2]);

	ALAP_DEBUG_L2(
	   "tx data: len(%d),data:(%x)(%x)(%x)(%x)(%x)(%x)(%x)..(%x)\n",
	   alap_rpc->trans_len,
	   alap_rpc->tx_tmpb[0],
	   alap_rpc->tx_tmpb[1],
	   alap_rpc->tx_tmpb[2],
	   alap_rpc->tx_tmpb[3],
	   alap_rpc->tx_tmpb[4],
	   alap_rpc->tx_tmpb[5],
	   alap_rpc->tx_tmpb[6],
	   alap_rpc->tx_tmpb[alap_rpc->trans_len - 2]);

	ALAP_DEBUG_L2(
	   "rx data: len(%ld),data:(%x)(%x)(%x)(%x)(%x)(%x)(%x)..(%x)\n",
	   status,
	   alap_rpc->rx_tmpb[0],
	   alap_rpc->rx_tmpb[1],
	   alap_rpc->rx_tmpb[2],
	   alap_rpc->rx_tmpb[3],
	   alap_rpc->rx_tmpb[4],
	   alap_rpc->rx_tmpb[5],
	   alap_rpc->rx_tmpb[6],
	   alap_rpc->rx_tmpb[alap_rpc->trans_len - 2]);

	/* copy tx data to retry buffer */
	memcpy(alap_rpc->retry.rty_buf,
		   alap_rpc->tx_tmpb,
		   alap_rpc->trans_len);

	/* check communication data length */
	if (unlikely(status != alap_rpc->trans_len))
		goto nack_response;
	/* check checksum */
	checkcs = alap_rpc_calc_checksum_chk(spi, alap_rpc->rx_tmpb,
	   (alap_rpc->trans_len - ALAP_RPC_OFFS_CSUM),
	   alap_rpc->rx_tmpb[alap_rpc->trans_len - ALAP_RPC_OFFS_CSUM]);
	if (unlikely(!checkcs))
		goto nack_response;
	/* Make ACK response */
	alap_rpc->tx_tmpb[ALAP_RPC_ACK_DATA] = ALAP_RPC_ACK;
	/* Callback data is Empty */
	if (alap_rpc->rx_tmpb[ALAP_RPC_INDX_NUMB] == 0)
		goto data_process_end;
	/* check rx buffer */
	if (!alap_rpc_read_buf_chk(alap_rpc)) {
		ALAP_DEBUG_L2(
		   "rx buffer full,rd_wt:%d,rd_rd:%d,rd_wt_p:%d,rd_rd_p:%d,bufc:%d\n",
		   alap_rpc->rd_wt.bit.index_c,
		   alap_rpc->rd_rd.bit.index_c,
		   alap_rpc->rd_wt.bit.page_bit,
		   alap_rpc->rd_rd.bit.page_bit,
		   alap_rpc->rd_buf_cont);
		goto nack_response;
	}
	/* copy rpc callback */
	ret = alap_rpc_rx_data_split(alap_rpc);
	if (ret)
		goto nack_response;
	/* Read wake up */
	if (alap_rpc->rd_block.condi == ALAP_RPC_WQ_WAIT) {
		alap_rpc->rd_block.condi = ALAP_RPC_WQ_WAKE;
		wake_up_interruptible(&alap_rpc->rd_block.wqueue);
	}
	goto data_process_end;
nack_response:
	/* Make NACK response */
	alap_rpc->tx_tmpb[ALAP_RPC_ACK_DATA] = ALAP_RPC_NACK;

data_process_end:
	alap_rpc->send_flag.process_flag = ALAP_RPC_SEND_ACK;

};

static void alap_rpc_ack_mesgchk(struct alap_rpc_data *alap_rpc)
{
	struct spi_device *spi = alap_rpc->spi_dev;
	ssize_t status = 0;

	/*
	 * Only this function can operation tx_read point
	 */

	/* communication ack/nack */
	status = alap_rpc_sync(alap_rpc, ALAP_RPC_ACK_SIZE);

	if ((alap_rpc->rx_tmpb[1] != 0) && (rpc_tdebug != 2))
		ALAP_DEBUG_L1(
		   "tx data: len(%d), tx_ACK(%02X)\n",
		   ALAP_RPC_ACK_SIZE,
		   alap_rpc->tx_tmpb[ALAP_RPC_ACK_DATA]);

	if ((alap_rpc->tx_tmpb[1] != 0) && (rpc_tdebug != 2))
		ALAP_DEBUG_L1(
		   "rx data: len(%ld), rx_ACK(%02X)\n",
		   status,
		   alap_rpc->rx_tmpb[ALAP_RPC_ACK_DATA]);

	ALAP_DEBUG_L2(
	   "tx data: len(%d), tx_ACK(%02X)\n",
	   ALAP_RPC_ACK_SIZE,
	   alap_rpc->tx_tmpb[ALAP_RPC_ACK_DATA]);

	ALAP_DEBUG_L2(
	   "rx data: len(%ld), rx_ACK(%02X)\n",
	   status,
	   alap_rpc->rx_tmpb[ALAP_RPC_ACK_DATA]);

	/* check ack/nack length */
	if (unlikely(status != ALAP_RPC_ACK_SIZE))
		goto nack_process;
	if (unlikely(alap_rpc->rx_tmpb[ALAP_RPC_ACK_DATA] != ALAP_RPC_ACK))
		goto nack_process;
	if (alap_rpc->send_flag.Empty_data_flag == ALAP_RPC_DEMPTY_FLAG)
		goto ack_process_end;
	alap_rpc->retry.count = 0;
	alap_rpc->wt_rd.bit.index_c += alap_rpc->send_flag.combi_count;
	if (unlikely(alap_rpc->wt_rd.bit.index_c >= alap_rpc->wt_buf_cont)) {
		alap_rpc->wt_rd.bit.index_c =
		   alap_rpc->wt_rd.bit.index_c - alap_rpc->wt_buf_cont;
		alap_rpc->wt_rd.bit.page_bit = ~alap_rpc->wt_rd.bit.page_bit;
	}
	goto ack_wait_wakeup;

nack_process:
	if (alap_rpc->send_flag.Empty_data_flag == ALAP_RPC_DEMPTY_FLAG)
		goto ack_process_end;
	if (alap_rpc->retry.count < ALAP_RPC_RETRY_MAX ) {
		alap_rpc->retry.count++;
		ALAP_DEBUG_L1("wt_rd(%d), retry (%d)\n",
		   alap_rpc->wt_rd.bit.index_c, alap_rpc->retry.count);
		goto ack_process_end;
	}
	alap_rpc->retry.count = 0;
	alap_rpc->wt_rd.bit.index_c += alap_rpc->send_flag.combi_count;
	if (alap_rpc->wt_rd.bit.index_c < alap_rpc->wt_buf_cont)
		goto ack_wait_wakeup;
	if (alap_rpc->wt_rd.bit.index_c >= alap_rpc->wt_buf_cont)
		alap_rpc->wt_rd.bit.index_c =
		   alap_rpc->wt_rd.bit.index_c - alap_rpc->wt_buf_cont;
	alap_rpc->wt_rd.bit.page_bit = ~alap_rpc->wt_rd.bit.page_bit;

ack_wait_wakeup:
	if (alap_rpc->wt_block.condi == ALAP_RPC_WQ_WAIT) {
		alap_rpc->wt_block.condi = ALAP_RPC_WQ_WAKE;
		wake_up_interruptible(&alap_rpc->wt_block.wqueue);
	}

ack_process_end:
	alap_rpc->send_flag.process_flag = ALAP_RPC_SEND_DATA;

};

static irqreturn_t alap_rpc_handle(int irq, void *dev_id)
{
	return IRQ_WAKE_THREAD;
}

static irqreturn_t alap_rpc_thread_fn(int irq, void *dev_id)
{
	struct alap_rpc_data *alap_rpc = dev_id;

	alap_rpc->trans_status = ALAP_RPC_TRANSING;

	if (alap_rpc->send_flag.process_flag == ALAP_RPC_SEND_DATA)
		alap_rpc_data_mesgchk(alap_rpc);
	else if (alap_rpc->send_flag.process_flag == ALAP_RPC_SEND_ACK)
		alap_rpc_ack_mesgchk(alap_rpc);

	if (alap_rpc->rpc_suspend.condi == ALAP_RPC_STR_WAIT) {
		alap_rpc->rpc_suspend.condi = ALAP_RPC_STR_WAKE;
		wake_up_interruptible(&alap_rpc->rpc_suspend.wqueue);
	}

	alap_rpc->trans_status = ALAP_RPC_NO_TRANS;

	return IRQ_HANDLED;
}

/* Write-only message with current device setup */
static ssize_t
alap_rpc_write(struct file *filp, const char __user *buf,
		size_t count, loff_t *f_pos)
{
	struct alap_rpc_data *alap_rpc	= filp->private_data;
	struct spi_device	 *spi		= alap_rpc->spi_dev;
	ssize_t				 status;
	unsigned long		 missing;
	unsigned int		 tmpbIndex;
	int ret;

	/*
	 * This function just operation tx_write point
	 */

	/* check input data length */
	if (count > alap_rpc->wt_len) {
		dev_warn(&spi->dev,
		    "write: trans len(%d) vs input len(%ld)\n", 
			alap_rpc->wt_len, count);
		status = alap_rpc->wt_len;
	} else {
		status = count;
	}

	mutex_lock(&alap_rpc->wt_buf_lock);

	ALAP_DEBUG_L1(
	   "write: wt_wt(%d) wx_wt_pag(%d) wt_rd(%d) wx_rd_pag(%d)\n",
	   alap_rpc->wt_wt.bit.index_c,
	   alap_rpc->wt_wt.bit.page_bit,
	   alap_rpc->wt_rd.bit.index_c,
	   alap_rpc->wt_rd.bit.page_bit);

	/* check read/write location index */
	if ((alap_rpc->wt_wt.bit.index_c == alap_rpc->wt_rd.bit.index_c)
	 && (alap_rpc->wt_wt.bit.page_bit !=
	   alap_rpc->wt_rd.bit.page_bit)) {
		ALAP_DEBUG_L1("write: blocking\n");
		alap_rpc->wt_block.condi = ALAP_RPC_WQ_WAIT;
		ret = wait_event_interruptible(
		   alap_rpc->wt_block.wqueue,
		   (alap_rpc->wt_block.condi == ALAP_RPC_WQ_WAKE));
		if (ret) {
			dev_warn(&spi->dev, "write: relase blocking ng\n");
			status = -ERESTARTSYS;
			goto err_write;
		}
	}

	/* Data copy from user land */
	tmpbIndex = alap_rpc->wt_wt.bit.index_c * alap_rpc->wt_len;
	missing = copy_from_user(
	   &(alap_rpc->wt_buf[tmpbIndex]),
	   buf, status);

	ALAP_DEBUG_L1(
	   "write: lnd(%d)data(%x)(%x)(%x)(%x)(%x)(%x)...(%x)(%ld)\n",
	   tmpbIndex,
	   alap_rpc->wt_buf[tmpbIndex],
	   alap_rpc->wt_buf[tmpbIndex + 1],
	   alap_rpc->wt_buf[tmpbIndex + 2],
	   alap_rpc->wt_buf[tmpbIndex + 3],
	   alap_rpc->wt_buf[tmpbIndex + 4],
	   alap_rpc->wt_buf[tmpbIndex + 5],
	   alap_rpc->wt_buf[tmpbIndex + status - 1],
	   missing);

	if (missing == 0) {
		/* write location index process */
		alap_rpc->wt_wt.bit.index_c++;
		if (alap_rpc->wt_wt.bit.index_c >= alap_rpc->wt_buf_cont) {
			alap_rpc->wt_wt.bit.index_c = 0;
			alap_rpc->wt_wt.bit.page_bit =
			   ~alap_rpc->wt_wt.bit.page_bit;
		}
	} else
		status = -EFAULT;

err_write:
	mutex_unlock(&alap_rpc->wt_buf_lock);

	return status;
}

/* Read-only message with current device setup */
static ssize_t
alap_rpc_read(struct file *filp, char __user *buf, size_t count, loff_t *f_pos)
{
	struct alap_rpc_data *alap_rpc	= filp->private_data;
	struct spi_device	 *spi		= alap_rpc->spi_dev;
	ssize_t				 status     = 0;
	unsigned long		 missing;
	unsigned int		 tmpbIndex;
	int ret;
	/*
	 * This function just operation rd_rd point
	 */

	mutex_lock(&alap_rpc->rd_buf_lock);


	ALAP_DEBUG_L1(
	   "read: rd_wt(%d) rd_wtag(%d) rd_rd(%d) rd_rdag(%d)\n",
	   alap_rpc->rd_wt.bit.index_c,
	   alap_rpc->rd_wt.bit.page_bit,
	   alap_rpc->rd_rd.bit.index_c,
	   alap_rpc->rd_rd.bit.page_bit);

	if ((alap_rpc->rd_rd.bit.index_c == alap_rpc->rd_wt.bit.index_c)
	 && (alap_rpc->rd_rd.bit.page_bit ==
	   alap_rpc->rd_wt.bit.page_bit)) {
		ALAP_DEBUG_L1("read: blocking\n");
		alap_rpc->rd_block.condi = ALAP_RPC_WQ_WAIT;
		ret = wait_event_interruptible(
		   alap_rpc->rd_block.wqueue,
		   (alap_rpc->rd_block.condi == ALAP_RPC_WQ_WAKE));
		if (ret) {
			dev_warn(&spi->dev, "read: relase blocking ng\n");
			status = -ERESTARTSYS;
			goto err_read;
		}
	}

	while (count >= alap_rpc->rd_len) {
		tmpbIndex = alap_rpc->rd_rd.bit.index_c * alap_rpc->rd_len;
		missing = copy_to_user(&buf[status],
							&(alap_rpc->rd_buf[tmpbIndex]),
							alap_rpc->rd_len);

		ALAP_DEBUG_L1(
			"read: lnd(%d)data(%x)(%x)(%x)(%x)(%x)(%x)...(%x)\n",
			tmpbIndex,
			alap_rpc->rd_buf[tmpbIndex],
			alap_rpc->rd_buf[tmpbIndex + 1],
			alap_rpc->rd_buf[tmpbIndex + 2],
			alap_rpc->rd_buf[tmpbIndex + 3],
			alap_rpc->rd_buf[tmpbIndex + 4],
			alap_rpc->rd_buf[tmpbIndex + 5],
			alap_rpc->rd_buf[tmpbIndex + alap_rpc->rd_len - 1]);

		if (missing == alap_rpc->rd_len) {
			status = -EFAULT;
			goto err_read;
		} else {
			alap_rpc->rd_rd.bit.index_c++;
			if (alap_rpc->rd_rd.bit.index_c >= alap_rpc->rd_buf_cont) {
				alap_rpc->rd_rd.bit.index_c = 0;
				alap_rpc->rd_rd.bit.page_bit =
					~alap_rpc->rd_rd.bit.page_bit;
			}
		}

		status = status + alap_rpc->rd_len;
		count = count - alap_rpc->rd_len;

		if ((alap_rpc->rd_rd.bit.index_c == alap_rpc->rd_wt.bit.index_c)
			&& (alap_rpc->rd_rd.bit.page_bit ==
			alap_rpc->rd_wt.bit.page_bit))
			break;
	}

err_read:
	mutex_unlock(&alap_rpc->rd_buf_lock);

	return status;
}

static void alap_rpc_initialization(struct alap_rpc_data *alap_rpc)
{
	/* Write/Read buffer */
	alap_rpc->rd_rd.index = 0;
	alap_rpc->rd_wt.index = 0;
	alap_rpc->wt_rd.index = 0;
	alap_rpc->wt_wt.index = 0;
	/* other */
	alap_rpc->send_flag.process_flag = ALAP_RPC_SEND_DATA;
	alap_rpc->retry.count = 0;
	alap_rpc->rd_block.condi = 0;
	alap_rpc->wt_block.condi = 0;
	alap_rpc->rpc_suspend.condi = 0;
	alap_rpc->trans_status = ALAP_RPC_NO_TRANS;
}

static int alap_rpc_open(struct inode *inode, struct file *filp)
{
	struct alap_rpc_data	*alap_rpc;
	struct spi_device		*spi;
	unsigned long			flags	= 0;
	int ret		= -ENXIO;

	mutex_lock(&device_list_lock);

	/* check open device */
	list_for_each_entry(alap_rpc, &device_list, device_entry) {
		if (alap_rpc->devt == inode->i_rdev) {
			ret = 0;
			break;
		}
	}

	spi = alap_rpc->spi_dev;

	if (ret) {
		dev_err(&spi->dev,
		   "open : nothing for minor %d\n",
		   iminor(inode));
		goto err_find_dev;
	}

	if (alap_rpc->users != 0) {
		ret = -EBUSY;
		goto err_find_dev;
	}

	alap_rpc_initialization(alap_rpc);

	/* irq thread request */
	if (!dev_fwnode(&spi->dev))
		flags = IRQF_TRIGGER_RISING;
#if 1 
{
	struct irq_desc * desc = irq_to_desc(spi->irq);
	if (desc) { 
		if (desc->irq_data.common) {
			struct cpumask mask;
			cpumask_clear(&mask);
			cpumask_set_cpu(4, &mask);
			irq_data_update_effective_affinity(&desc->irq_data, &mask);
			dev_info(&spi->dev, "alap_rpc_open set effective affinity %d \n", spi->irq);
		} else {
			dev_info(&spi->dev, "alap_rpc_open set effective affinity no common %d\n", spi->irq);
		}
	} else {
		dev_info(&spi->dev, "alap_rpc_open set effective affinity get desc fail %d\n", spi->irq);
	}
}
#endif

	ret = request_threaded_irq(spi->irq,
	   alap_rpc_handle, alap_rpc_thread_fn,
	   flags, DEVICE_NAME, alap_rpc);

	if (ret) {
		dev_err(&spi->dev,
		   "open err: irq request(%d)\n",
		   spi->irq);
		goto err_find_dev;
	}

	/* check start gpio */
	if (!IS_ERR_OR_NULL(alap_rpc->startup_gpiod)) {
		ALAP_DEBUG_L1(
		   "open: get startup gpio value(%d)\n",
		   gpiod_get_value(alap_rpc->startup_gpiod));

		gpiod_set_value(alap_rpc->startup_gpiod, GPIO_HIGH);

		ALAP_DEBUG_L1(
		   "open: get startup gpio value(%d)\n",
		   gpiod_get_value(alap_rpc->startup_gpiod));
	} else
		dev_err(&spi->dev, 
	        "open: no startup gpio, irq start\n");

	filp->private_data = alap_rpc;
	alap_rpc->users++;


err_find_dev:
	mutex_unlock(&device_list_lock);

	return ret;
}

static int alap_rpc_release(struct inode *inode, struct file *filp)
{
	struct alap_rpc_data *alap_rpc = filp->private_data;
	struct spi_device *spi = alap_rpc->spi_dev;

	mutex_lock(&device_list_lock);

	free_irq(spi->irq, alap_rpc);

	if (!IS_ERR_OR_NULL(alap_rpc->startup_gpiod)) {
		ALAP_DEBUG_L1(
		   "release: get startup gpio value(%d)\n",
		   gpiod_get_value(alap_rpc->startup_gpiod));

		gpiod_set_value(alap_rpc->startup_gpiod, GPIO_LOW);

		ALAP_DEBUG_L1(
		   "release: get startup gpio value(%d)\n",
		   gpiod_get_value(alap_rpc->startup_gpiod));
	} else
		dev_err(&spi->dev, 
	        "release: no startup gpio, irq stop\n");

	alap_rpc->users = (alap_rpc->users > 0) ? (--alap_rpc->users) : alap_rpc->users;

	mutex_unlock(&device_list_lock);

	return 0;
}

static const struct file_operations alap_rpc_fops = {
	.owner =	THIS_MODULE,
	.write =	alap_rpc_write,
	.read =		alap_rpc_read,
	.open =		alap_rpc_open,
	.release =	alap_rpc_release,
	.llseek =	no_llseek,
};

/*-------------------------------------------------------------------------*/

/* The main reason to have this class is to make mdev/udev create the
 * /dev/alap_rpcB.C character device nodes exposing our userspace API.
 * It also simplifies memory management.
 */

static struct class *alap_rpc_class;

#ifdef CONFIG_OF
static const struct of_device_id alap_rpc_dt_ids[] = {
	{ .compatible = "alap,ucom_rpc" },
	{},
};
MODULE_DEVICE_TABLE(of, alap_rpc_dt_ids);
#endif
/*-------------------------------------------------------------------------*/
static int alap_rpc_config_info_imp(struct alap_rpc_data *alap_rpc)
{
	struct spi_device	*spi;
	int					ret		= 0;

	if ((!alap_rpc) || (!alap_rpc->spi_dev)) {
		return -EINVAL;
	}
	spi	= alap_rpc->spi_dev;

	/* alap_trans_len */
	alap_rpc->trans_len = ALAP_RPC_DATA_SIZE;
	if (alap_rpc->trans_len <= ALAP_RPC_FRAME_SIZE) {
		dev_err(&spi->dev,
		   "probe err: tran length less than frame length(%d)\n",
		   ALAP_RPC_FRAME_SIZE);
		ret = -EINVAL;
		goto get_config_imp_err;
	} else {
		dev_info(&spi->dev, "trans lens is (%d)\n",
		   alap_rpc->trans_len);
	}
	/* alap_wt_buf_cont */
	alap_rpc->wt_buf_cont = ALAP_RPC_WT_COUNT;
	if (alap_rpc->wt_buf_cont == 0) {
		dev_err(&spi->dev, "probe err: write buffer count is (%d)\n",
		   alap_rpc->wt_buf_cont);
		ret = -EINVAL;
		goto get_config_imp_err;
	} else
		dev_info(&spi->dev, "write buffer cont is (%d)\n",
		   alap_rpc->wt_buf_cont);

	/* alap_wt_len */
	alap_rpc->wt_len = ALAP_RPC_WT_LEN;
	if (alap_rpc->wt_len == 0) {
		dev_err(&spi->dev, "probe err: write length is (%d)\n",
		   alap_rpc->wt_len);
		ret = -EINVAL;
		goto get_config_imp_err;
	} else if (alap_rpc->wt_len > alap_rpc->trans_len) {
		dev_err(&spi->dev, "probe err: wt len longer than trans len (%d) vs (%d)\n",
		   alap_rpc->wt_len, alap_rpc->trans_len);
		ret = -EINVAL;
		goto get_config_imp_err;
	} else
		dev_info(&spi->dev, "write buffer length is(%d)\n",
		   alap_rpc->wt_len);

	/* alap_rd_buf_cont */
	alap_rpc->rd_buf_cont = ALAP_RPC_RD_COUNT;
	if (alap_rpc->rd_buf_cont == 0) {
		dev_err(&spi->dev, "probe err: read buffer count is (%d)\n",
		   alap_rpc->rd_buf_cont);
		ret = -EINVAL;
		goto get_config_imp_err;
	} else
		dev_info(&spi->dev, "read buffer cont is (%d)\n",
		   alap_rpc->rd_buf_cont);

	/* alap_rd_len */
	alap_rpc->rd_len = ALAP_RPC_RD_LEN;
	if (alap_rpc->rd_len == 0) {
		dev_err(&spi->dev, "probe err: read length is (%d)\n",
		   alap_rpc->rd_len);
		ret = -EINVAL;
		goto get_config_imp_err;
	} else if (alap_rpc->rd_len > alap_rpc->trans_len) {
		dev_err(&spi->dev, "probe err: rd len longer than trans len (%d) vs (%d)\n",
		   alap_rpc->rd_len, alap_rpc->trans_len);
		ret = -EINVAL;
		goto get_config_imp_err;
	} else
		dev_info(&spi->dev, "read buffer length is (%d)\n",
		   alap_rpc->rd_len);

get_config_imp_err:
	return ret;
}

static int alap_rpc_config_info_np(struct alap_rpc_data *alap_rpc)
{
	struct spi_device *spi = alap_rpc->spi_dev;

	/* alap_tx_combi */
	dev_info(&spi->dev, "tx data combi configuration = %d\n", ALAP_RPC_DATA_COMBI);

	/* alap_rx_split */
	dev_info(&spi->dev, "rx data split configuration = %d\n", ALAP_RPC_DATA_SPLIT);

	/* alap_retry_max */
	dev_info(&spi->dev, "retry max counter(%d)\n", ALAP_RPC_RETRY_MAX );

	return 0;
}

static int alap_rpc_buffer_alloc(struct alap_rpc_data *alap_rpc)
{
	struct spi_device *spi = alap_rpc->spi_dev;
	unsigned int tmplen;

	/* allocate tx tmp buffer */
	tmplen = alap_rpc->trans_len;
	alap_rpc->tx_tmpb = devm_kzalloc(&spi->dev, tmplen, GFP_KERNEL);
	if (!alap_rpc->tx_tmpb) {
		dev_err(&spi->dev, "probe err: tx_tmpb null\n");
		return -ENOMEM;
	}

	/* allocate rx tmp buffer */
	tmplen = alap_rpc->trans_len;
	alap_rpc->rx_tmpb = devm_kzalloc(&spi->dev, tmplen, GFP_KERNEL);
	if (!alap_rpc->rx_tmpb) {
		dev_err(&spi->dev, "probe err: rx_tmpb null\n");
		return -ENOMEM;
	}

	/* allocate user process write buffer */
	tmplen = alap_rpc->wt_len * alap_rpc->wt_buf_cont;
	alap_rpc->wt_buf = devm_kzalloc(&spi->dev, tmplen, GFP_KERNEL);
	if (!alap_rpc->wt_buf) {
		dev_err(&spi->dev, "probe err: wt_buffer null\n");
		return -ENOMEM;
	}

	/* allocate user process read buffer */
	tmplen = alap_rpc->rd_len * alap_rpc->rd_buf_cont;
	alap_rpc->rd_buf = devm_kzalloc(&spi->dev, tmplen, GFP_KERNEL);
	if (!alap_rpc->rd_buf) {
		dev_err(&spi->dev, "probe err: rd_buffer null\n");
		return -ENOMEM;
	}

	/* allocate retry buffer */
	tmplen = alap_rpc->trans_len;
	alap_rpc->retry.rty_buf = devm_kzalloc(&spi->dev, tmplen, GFP_KERNEL);
	if (!alap_rpc->retry.rty_buf) {
		dev_err(&spi->dev, "probe err: retry buffer null\n");
		return -ENOMEM;
	}

	return 0;
}

static int alap_rpc_probe(struct spi_device *spi)
{
	struct alap_rpc_data *alap_rpc;
	struct device *dev;
	unsigned long minor;
	int ret = 0;

	/* allocate driver data */
	alap_rpc = devm_kzalloc(&spi->dev,
	   sizeof(struct alap_rpc_data), GFP_KERNEL);
	if (!alap_rpc) {
		return -ENOMEM;
	}

	/* initialize the driver data */
	alap_rpc->spi_dev = spi;
	spin_lock_init(&alap_rpc->spi_lock);
	mutex_init(&device_list_lock);
	mutex_init(&alap_rpc->wt_buf_lock);
	mutex_init(&alap_rpc->rd_buf_lock);

	/* configuration for impact on probe */
	ret = alap_rpc_config_info_imp(alap_rpc);
	if (ret)
		goto error_ret;
	/* get information from dts(No impact on probe) */
	alap_rpc_config_info_np(alap_rpc);

	/* configure the SPI bus  */
	spi->bits_per_word = 8;
	spi->max_speed_hz = ALAP_RPC_MAXSPEEDHZ;
	spi->mode = SPI_MODE_3;
	dev_info(&spi->dev, "max_speed_hz(%d), mode(%X)\n",
	   spi->max_speed_hz, spi->mode);
	ret = spi_setup(spi);
	if (ret) {
		dev_err(&spi->dev, "probe err: spi config\n");
		goto error_ret;
	}

	/* allocate memory */
	alap_rpc_buffer_alloc(alap_rpc);

	spi_set_drvdata(spi, alap_rpc);

	/* rpc process start up conctrl */
	alap_rpc->startup_gpiod = devm_gpiod_get_optional(&spi->dev,
	   "startup", GPIOD_OUT_LOW);
	if (!alap_rpc->startup_gpiod) {
		dev_err(&spi->dev, "no start up gpio, request irq at file open\n");
		alap_rpc->startup_gpiod = NULL;
	}

	/* initialize objects for exclusive access. */
	init_waitqueue_head(&alap_rpc->wt_block.wqueue);
	init_waitqueue_head(&alap_rpc->rd_block.wqueue);

	/* creat file device */
	mutex_lock(&device_list_lock);
	minor = find_first_zero_bit(minors, ALAP_RPC_N_SPI_MINORS);
	if (minor < ALAP_RPC_N_SPI_MINORS) {
		alap_rpc->devt = MKDEV(ALAP_RPC_MAJOR, minor);
		dev = device_create(alap_rpc_class, &spi->dev, alap_rpc->devt,
		   alap_rpc, "alap_rpc");
		ret = PTR_ERR_OR_ZERO(dev);
	} else {
		dev_err(&spi->dev, "no minor number available!\n");
		ret = -ENODEV;
	}
	if (ret == 0) {
		set_bit(minor, minors);
		list_add(&alap_rpc->device_entry, &device_list);
	}
	mutex_unlock(&device_list_lock);

	INIT_LIST_HEAD(&alap_rpc->device_entry);

	rpc_tdebug = RPC_DEBUG_LEVEL;

	alap_rpc->users = 0;

	/* probe end */
	dev_info(&spi->dev, "probe successful\n");

error_ret:
	dev_info(&spi->dev, "alap_rpc probe end (%d)\n", ret);
	return ret;
}

static int alap_rpc_remove(struct spi_device *spi)
{
	struct alap_rpc_data	*alap_rpc = spi_get_drvdata(spi);

	/* make sure ops on existing fds can abort cleanly */
	spin_lock_irq(&alap_rpc->spi_lock);
	alap_rpc->spi_dev = NULL;
	spin_unlock_irq(&alap_rpc->spi_lock);

	/* prevent new opens */
	mutex_lock(&device_list_lock);
	device_destroy(alap_rpc_class, alap_rpc->devt);
	mutex_unlock(&device_list_lock);

	return 0;
}

static int __maybe_unused alap_rpc_suspend(struct device *dev)
{
	struct spi_device *spi;
	struct alap_rpc_data *alap_rpc;

	spi = to_spi_device(dev);
	if (IS_ERR_OR_NULL(spi)) {
		printk("suspend spi is NULL");
		return 0;
	}
	alap_rpc = spi_get_drvdata(spi);

	disable_irq(spi->irq);

	if (!IS_ERR_OR_NULL(alap_rpc->startup_gpiod))
		gpiod_set_value(alap_rpc->startup_gpiod, GPIO_LOW);

	if (alap_rpc->trans_status == ALAP_RPC_TRANSING) {
		alap_rpc->rpc_suspend.condi = ALAP_RPC_STR_WAIT;
		wait_event_interruptible(alap_rpc->rpc_suspend.wqueue,
		   (alap_rpc->rpc_suspend.condi == ALAP_RPC_STR_WAKE));
	}

	alap_rpc_initialization(alap_rpc);

	ALAP_DEBUG_L1(
	   "suspend: startup gpio (%d)\n",
	   gpiod_get_value(alap_rpc->startup_gpiod));

	pinctrl_pm_select_sleep_state(dev);

	return 0;
}

static int __maybe_unused alap_rpc_resume(struct device *dev)
{
	struct spi_device *spi;
	struct alap_rpc_data *alap_rpc;

	spi = to_spi_device(dev);
	if (IS_ERR_OR_NULL(spi)) {
		printk("resume spi is NULL");
		return 0;
	}
	alap_rpc = spi_get_drvdata(spi);

	pinctrl_pm_select_default_state(dev);

	enable_irq(spi->irq);

	if (!IS_ERR_OR_NULL(alap_rpc->startup_gpiod))
		gpiod_set_value(alap_rpc->startup_gpiod, GPIO_HIGH);

	ALAP_DEBUG_L1(
	   "resume: startup gpio (%d)\n",
	   gpiod_get_value(alap_rpc->startup_gpiod));

	return 0;
}

static SIMPLE_DEV_PM_OPS(alap_rpc_pm_ops, alap_rpc_suspend,
	alap_rpc_resume);

static struct spi_driver alap_rpc_spi_driver = {
	.driver = {
		.name =		"alap_rpc",
		.of_match_table = of_match_ptr(alap_rpc_dt_ids),
		.pm = &alap_rpc_pm_ops,
	},
	.probe =	alap_rpc_probe,
	.remove =	alap_rpc_remove,
};

static int __init alap_rpc_init(void)
{
	int status;

	BUILD_BUG_ON(ALAP_RPC_N_SPI_MINORS > 256);

	status = register_chrdev(ALAP_RPC_MAJOR, "spi", &alap_rpc_fops);
	if (status < 0)
		return status;

	alap_rpc_class = class_create(THIS_MODULE, "alap_rpc");
	if (IS_ERR(alap_rpc_class)) {
		unregister_chrdev(ALAP_RPC_MAJOR,
		   alap_rpc_spi_driver.driver.name);
		return PTR_ERR(alap_rpc_class);
	}

	status = spi_register_driver(&alap_rpc_spi_driver);
	if (status < 0) {
		class_destroy(alap_rpc_class);
		unregister_chrdev(ALAP_RPC_MAJOR,
		   alap_rpc_spi_driver.driver.name);
	}
	return status;
}
module_init(alap_rpc_init);

static void __exit alap_rpc_exit(void)
{
	spi_unregister_driver(&alap_rpc_spi_driver);
	class_destroy(alap_rpc_class);
	unregister_chrdev(ALAP_RPC_MAJOR, alap_rpc_spi_driver.driver.name);
}
module_exit(alap_rpc_exit);

MODULE_AUTHOR("xxxxxxxx, <xxxxxx@neusoft.com>");
MODULE_DESCRIPTION("User mode SPI device interface");
MODULE_LICENSE("GPL");
MODULE_ALIAS("spi:alap_rpc");
